/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Nov 26, 2006
 *
 */

#include "bitset.h"

#define _bit(b) (1<<((b)&7))

/**
 * Set specified bit to 1
 *
 * @param BitSet*, The pointer to the BitSet
 * @param unsigned char, The bit index
 */
void set_bit(BitSet* bitset, unsigned char b){
  *(bitset->data+(b>>3)) |= _bit(b);
}

/**
 * Set specified bit to 0
 *
 * @param BitSet*, The pointer to the BitSet
 * @param unsigned char, The bit index
 */
void clr_bit(BitSet* bitset, unsigned char b){
  *(bitset->data+(b>>3)) &= ~_bit(b);
}

/**
 * Filp specified bit
 *
 * @param BitSet*, The pointer to the BitSet
 * @param unsigned char, The bit index
 */
void filp_bit(BitSet* bitset, unsigned char b){
  *(bitset->data+(b>>3)) ^= _bit(b);
}

/**
 * Clear all bits to 0
 *
 * @param BitSet*, The pointer to the BitSet
 * @param unsigned char, The bit index
 */
void clear_bits(BitSet* bitset){
  /* Why not clear byte directly? 
     Because I won't enlarge code size */
  for(unsigned char i=0; i<bitset->bits; i++){
    clr_bit(bitset, i);
  }
}

/**
 * Get specified bit
 *
 * @param BitSet*, The pointer to the BitSet
 * @param unsigned char, The bit index
 * @return bool the bit value
 */
bool get_bit(BitSet* bitset, unsigned char b){
  return (*(bitset->data+(b>>3)) & _bit(b)) ? TRUE : FALSE;
}

/**
 * Return the number of bits set to 1
 *
 * @param BitSet*, The pointer to the BitSet
 * @return unsigned char, The number of bits set to 1
 */
unsigned char count_bits(BitSet* bitset){
  unsigned char n = 0;
  /* Why not consider other high performance algorithms?
     Because I won't enlarge code size */
  for(unsigned char i=0; i<bitset->bits; i++){
    if (get_bit(bitset, i)) n++;
  }
  return n;
}

