/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 27, 2006
 */

#include "display.h"
#include "avrcx.h"

#if __DISPLAY__ == 1

CHARDISPLAY(disp, DISPLAY_COLS, DISPLAY_ROWS);

/**
 * Initialize display module
 */
void init_display(Driver* drv){
  disp.driver = drv;
  disp.driver->_drv_init_hardware();
  //  disp_clean();
}

/**
 * Move cursor to new site
 *
 * @param unsigned char y, The point y
 * @param unsigned char x, The point x
 * @return Point*, The pointer to the cursor struct
 */
void disp_moveto(unsigned char y, unsigned char x){
  disp.x = (x < disp.cols) ? x : disp.cols-1;
  disp.y = (y < disp.rows) ? y : disp.rows-1;
}

/**
 * Write a character to display buffer of current cursor
 *
 * @param unsigned char b, The character
 */
INTERFACE void disp_putchar(unsigned char b){

  #if defined DISPLAY_CRLF
  if (b == '\r') disp.x = 0;
  if (b == '\n') 
    disp.y = (disp.y < disp.rows-1) ? disp.y+1 : 0;
  #endif

  *(disp.data + disp.x + disp.y*disp.cols) = b;

  disp.x++;
  if (disp.x >= disp.cols){
    disp.x = 0;
    disp.y = (disp.y < disp.rows-1) ? disp.y+1 : 0;
  }
}

/**
 * Retrives the character from display buffer of current cursor
 *
 * @return unsigned char, The character
 */
INTERFACE unsigned char disp_getchar(void){
  return   *(disp.data + disp.x + disp.y*disp.cols);
}

/**
 * Clear display buffer
 */
INTERFACE void disp_clean(void){
  for(unsigned int i = 0; i < DISPLAY_BUFSIZE; i++){
    *(disp.data+i) = DISPLAY_BLANK;
  }
}

/**
 * Write a character to specified site
 *
 * @param unsigned char row, The row number
 * @param unsigned char col, The col number
 * @param unsigned char b, The character
 */
INTERFACE void disp_write(unsigned char row, unsigned char col, 
			  unsigned char b){
  *(disp.data + col + row*disp.cols) = b;
}

/**
 * Retrives the character from display buffer with specified site
 * This method won't change cursor
 *
 * @param unsigned char row
 * @param unsigned char col
 * @return unsigned char, The character
 */
INTERFACE unsigned char disp_read(unsigned char row, unsigned char col){
  return *(disp.data + col + row*disp.cols);
}

/**
 * Refresh display buffer
 * User should invokes this method in timer to reflash screen
 */
INTERFACE void disp_refresh(void){
  _disp_flush(disp)(disp.rows, disp.cols, disp.data);
}

#endif /* __DISPLAY__ == 1 */
