/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 27, 2006
 */


#ifndef __DISPLAY_H__
#define __DISPLAY_H__

#include "avrcx.h"

#define DISPLAY_BUFSIZE DISPLAY_ROWS*DISPLAY_COLS

#ifndef __ASSEMBLER__
/* These only work in C program */

typedef struct CharacterBuffer {
  __volatile__ unsigned char cols;
  __volatile__ unsigned char rows;
  __volatile__ unsigned char x;
  __volatile__ unsigned char y;
  unsigned char *data;
  Driver* driver;
} CharacterBuffer;

#define CHARDISPLAY(name, cols, rows)      \
  unsigned char name##_data[cols*rows];    \
  CharacterBuffer name = {                 \
    cols, rows,   			   \
    0, 0,                                  \
    &name##_data[0],                       \
    NULL                                   \
  }

/**
 * Initialize display module
 *
 * @param Driver*, The pointer to the display driver
 */
INTERFACE void init_display(Driver*);

/**
 * Move cursor to new site
 *
 * @param unsigned char y, The point y
 * @param unsigned char x, The point x
 * @return Point*, The pointer to the cursor struct
 */
INTERFACE void disp_moveto(unsigned char y, unsigned char x);

/**
 * Write a character to display buffer of current cursor
 *
 * @param unsigned char b, The character
 */
INTERFACE void disp_putchar(unsigned char b);

/**
 * Retrives the character from display buffer of current cursor
 *
 * @return unsigned char, The character
 */
INTERFACE unsigned char disp_getchar(void);

/**
 * Cleanup display buffer
 */
INTERFACE void disp_clean(void);

/**
 * Write a character to specified site
 *
 * @param unsigned char row, The row number
 * @param unsigned char col, The col number
 * @param unsigned char b, The character
 */
INTERFACE void disp_write(unsigned char row, unsigned char col, 
			  unsigned char b);

/**
 * Retrives the character from display buffer with specified site
 * This method won't change cursor
 *
 * @param unsigned char row
 * @param unsigned char col
 * @return unsigned char, The character
 */
INTERFACE unsigned char disp_read(unsigned char row, unsigned char col);

/**
 * Refresh display buffer
 * User should invokes this method in timer to reflash screen, 
 * For example, 10ms
 */
INTERFACE void disp_refresh(void);

/* Driver need implements methods */
typedef void (*drv_disp_write)(unsigned char row, unsigned char col, 
			       unsigned char b);

typedef void (*drv_disp_flush)(unsigned char rows, unsigned cols,  
				 unsigned char* pBuf);

#define _disp_write(D)  ((drv_disp_write)((D).driver->drv_methods[0]))
#define _disp_flush(D)  ((drv_disp_flush)((D).driver->drv_methods[1]))


#endif /* !__ASSEMBLER__ */


#endif /* __DISPLAY_H__ */



