/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 26, 2006
 */


#ifndef __KEYBOARD_H__
#define __KEYBOARD_H__

#include "avrcx.h"


#ifndef __ASSEMBLER__
/* These only work in C program */

typedef struct Keyboard {
  __volatile__ unsigned char status;
  __volatile__ unsigned char mask;
  Driver* kbddrv;    // Local keyboad
  Driver* ircdrv;    // Infrared remote controller
} Keyboard;

#define KEYBOARD(name) Keyboard name = {0, 0xFF, NULL, NULL}


INTERFACE void init_keyboard(Driver* kbd, Driver* irc);

/**
 * Scan keyboard, this method should be invoked every 30ms
 *
 * @return unsigned char, The key value, if 0xFF, no key down
 */
INTERFACE unsigned char kbd_getchar(void);

typedef unsigned char (*drv_kbd_getchar)(void);

#define _kbd_getchar(K) ((drv_kbd_getchar)((K).kbddrv->drv_methods[0]))
#define _irc_getchar(K) ((drv_kbd_getchar)((K).ircdrv->drv_methods[0]))

#endif /* ! __ASSEMBLER__ */

#endif /* __KEYBOARD_H__ */
