/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Jan 3, 2007
 */


#ifndef __SPI_H__
#define __SPI_H__

#include "avrcx.h"

#ifndef __ASSEMBLER__
/* These only work in C program */

typedef struct SPIControlBlock {
  unsigned char *txbuf;
  unsigned char *rxbuf;
  unsigned char count;
  unsigned char point;
  Event *spi_complete;
} SPICB;

/**
 * Initialize SPI for Rx and Tx
 * You should invoke this method to initializ SPI
 *
 */
INTERFACE void init_spi(void);

/**
 * Start SPI to send and recv data
 * This method will blocked until all data are sent
 *
 * @param unsigned char* txBuf, The pointer to the send buffer
 * @param unsigned char* rxBuf, The pointer to the recv buffer
 * @param unsigned char size, buffer length
 */
INTERFACE void spi_write(unsigned char* txBuf, unsigned char* rxBuf, 
		     unsigned char size);
/**
 * Fixed SPI Tx and Rx buffer
 *
 * @param unsigned char* txBuf, The pointer to the send buffer
 * @param unsigned char* rxBuf, The pointer to the recv buffer
 * @param unsigned char size, buffer length
 */
INTERFACE void spi_setbuf(unsigned char* txBuf, unsigned char* rxBuf,
			  unsigned char size);

/**
 * Start SPI
 * This method will blocked until all data are sent
 */
INTERFACE void spi_start(void);

#endif /* ! __ASSEMBLER__ */

#endif /* __SPI_H__ */
