/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Nov 26, 2006
 */


#include "util.h"
#include <avr/pgmspace.h>

const prog_uchar hexasc[] = "0123456789ABCDEF";

union16 hex2bcd(unsigned char b){
  union32 r = {0};
  r.b2 = b/100;
  r.b1 = b/10 - r.b2*10;
  r.b0 = b - r.b2*100 - r.b1*10;

  // Pack BCD
  r.b1 = r.b1 << 4;
  r.b0 |= r.b1;
  r.b1 = r.b2;

  return r.lo16;
}

union16 hex2asc(unsigned char b){
  union16 r = {0};
  r.b0 = pgm_read_byte(&hexasc[b & 0x0F]);
  r.b1 = pgm_read_byte(&hexasc[b >> 4]);
  return r;
}

/**
 * 0~255 
 */
unsigned char asc2hex(union16 asc){
  unsigned char b = (asc.b1 <= 0x39) ? 
    (asc.b1 - 0x30)*10 : (asc.b1 - 0x37)*10;
  b += (asc.b0 <= 0x39) ? (asc.b0 - 0x30) : (asc.b0 - 0x37);
  return b;
}

union32 bcd2asc(union16 bcd){
  union32 asc = {0};
  asc.hi16 = hex2asc(bcd.hi8);
  asc.lo16 = hex2asc(bcd.lo8);
  return asc;
}

/* 0~255 */
union16 asc2bcd(union32 asc){
  union16 bcd = {0};
  bcd.b1 = (asc.b2 != 0) ? (asc.b2 - 0x30) : 0;
  bcd.b0 = (asc.b1 != 0) ? (asc.b1 - 0x30)<<4 : 0;
  bcd.b0 |=((asc.b0 != 0)? (asc.b0 - 0x30) : 0);
  return bcd;
}

/* 0~255 */
unsigned char bcd2hex(union16 bcd){
  unsigned char b = bcd.b1*100;
  b += (bcd.b0 >> 4)*10;
  b += (bcd.b0 & 0xF);
  return b;
}
